@@ -15,9 +15,8 @@ from six.moves import queue as Queue
 import sys
 import threading
 
-from awscli.customizations.s3.tasks import BasicTask
-from awscli.customizations.s3.utils import MultiCounter, NoBlockQueue, \
-    uni_print
+from awscli.customizations.s3.utils import NoBlockQueue, uni_print
+
 
 LOGGER = logging.getLogger(__name__)
 
@@ -29,34 +28,36 @@ class Executer(object):
     ``Executer``runs is a worker and a print thread.
     """
     def __init__(self, done, num_threads, timeout,
-                 print_queue, quiet, interrupt, max_multi, max_queue_size):
+                 result_queue, quiet, interrupt, max_queue_size):
         self.queue = None
         self.done = done
         self.num_threads = num_threads
         self.timeout = timeout
-        self.print_queue = print_queue
+        self.result_queue = result_queue
         self.quiet = quiet
         self.interrupt = interrupt
         self.threads_list = []
-        self.max_multi = max_multi
-        self.multi_lock = threading.Lock()
-        self.multi_counter = MultiCounter()
         self._max_queue_size = max_queue_size
+        self.print_thread = None
+
+    @property
+    def num_tasks_failed(self):
+        tasks_failed = 0
+        if self.print_thread is not None:
+            tasks_failed = self.print_thread.num_errors_seen
+        return tasks_failed
 
     def start(self):
-        self.queue = NoBlockQueue(self.interrupt, maxsize=self._max_queue_size)
-        self.multi_counter.count = 0
-        self.print_thread = PrintThread(self.print_queue, self.done,
+        self.print_thread = PrintThread(self.result_queue, self.done,
                                         self.quiet, self.interrupt,
                                         self.timeout)
-        self.print_thread.setDaemon(True)
+        self.print_thread.daemon = True
+        self.queue = NoBlockQueue(self.interrupt, maxsize=self._max_queue_size)
         self.threads_list.append(self.print_thread)
         self.print_thread.start()
         for i in range(self.num_threads):
             worker = Worker(queue=self.queue, done=self.done,
-                            timeout=self.timeout, multi_lock=self.multi_lock,
-                            multi_counter=self.multi_counter,
-                            max_multi=self.max_multi)
+                            timeout=self.timeout)
             worker.setDaemon(True)
             self.threads_list.append(worker)
             worker.start()
@@ -65,6 +66,7 @@ class Executer(object):
         """
         This is the function used to submit a task to the ``Executer``.
         """
+        LOGGER.debug("Submitting task: %s", task)
         self.queue.put(task)
 
     def wait(self):
@@ -85,70 +87,30 @@ class Executer(object):
 class Worker(threading.Thread):
     """
     This thread is in charge of performing the tasks provided via
-    the main queue ``queue``.  Across all ``worker`` threads, there can
-    only be the ``max_multi`` amount of multipart operations at a time or
-    deadlock occurs because ``worker`` threads are needed to perform the
-    part operations of multipart operations.  The ``worker`` threads share
-    a lock in order to accurately track these multipart operations.
+    the main queue ``queue``.
     """
-    def __init__(self, queue, done, timeout, multi_lock,
-                 multi_counter, max_multi):
+    def __init__(self, queue, done, timeout):
         threading.Thread.__init__(self)
+        # This is the queue where work (tasks) are submitted.
         self.queue = queue
         self.done = done
         self.timeout = timeout
-        self.multi_lock = multi_lock
-        self.multi_counter = multi_counter
-        self.max_multi = max_multi
-        self.is_multi_start = False
 
     def run(self):
         while True:
             try:
                 function = self.queue.get(True, self.timeout)
-                with self.multi_lock:
-                    putback = self.check_multi(function)
-                if not putback:
-                    try:
-                        function()
-                    except Exception as e:
-                        LOGGER.debug('Error calling task: %s', e,
-                                     exc_info=True)
-                else:
-                    self.queue.put(function)
-                if self.is_multi_start:
-                    with self.multi_lock:
-                        self.multi_counter.count -= 1
+                try:
+                    function()
+                except Exception as e:
+                    LOGGER.debug('Error calling task: %s', e,
+                                    exc_info=True)
                 self.queue.task_done()
             except Queue.Empty:
                 pass
             if self.done.isSet():
                 break
 
-    def check_multi(self, function):
-        """
-        This is a helper function used to handle the number of in progress
-        multipart operations.  The function determines if the number of
-        multipart operations in progress ``multi_counter.count`` is equal
-        to the maximum ``max_multi``.  If it is not equal it increments
-        ``multi_counter.count`` and allows the thread to proceed.
-        However if it is at the limit, The thread puts the task back into
-        the queue so that it performs a task that is not initiating a
-        multipart operation.
-        """
-        self.is_multi_start = False
-        putback = False
-        if isinstance(function, BasicTask):
-            if hasattr(function.filename, 'is_multi'):
-                self.is_multi_start = function.filename.is_multi
-            is_maxed = self.multi_counter.count == self.max_multi
-            if is_maxed and self.is_multi_start:
-                putback = True
-                self.is_multi_start = False
-            elif not is_maxed and self.is_multi_start:
-                self.multi_counter.count += 1
-        return putback
-
 
 class PrintThread(threading.Thread):
     """
@@ -156,12 +118,25 @@ class PrintThread(threading.Thread):
     completely finished it is permanently write the result to standard
     out. Otherwise, it is a part of a multipart upload/download and
     only shows the most current part upload/download.
+
+    Result Queue
+    ------------
+
+    Result queue items are dictionaries that have the following keys:
+
+        * message: An arbitrary string associated with the entry.   This
+            can be used to communicate the result of the task.
+        * error: Boolean indicating whether or not the task completely
+            successfully.
+        * total_parts: The total number of parts for multipart transfers (
+            deprecated, will be removed in the future).
+
     """
-    def __init__(self, print_queue, done, quiet, interrupt, timeout):
+    def __init__(self, result_queue, done, quiet, interrupt, timeout):
         threading.Thread.__init__(self)
         self._progress_dict = {}
         self._interrupt = interrupt
-        self._print_queue = print_queue
+        self._result_queue = result_queue
         self._done = done
         self._quiet = quiet
         self._timeout = timeout
@@ -173,6 +148,10 @@ class PrintThread(threading.Thread):
         self._total_parts = 0
         self._total_files = '...'
 
+        # This is a public attribute that clients can inspect to determine
+        # whether or not we saw any results indicating that an error occurred.
+        self.num_errors_seen = 0
+
     def set_total_parts(self, total_parts):
         with self._lock:
             self._total_parts = total_parts
@@ -184,7 +163,8 @@ class PrintThread(threading.Thread):
     def run(self):
         while True:
             try:
-                print_task = self._print_queue.get(True, self._timeout)
+                print_task = self._result_queue.get(True, self._timeout)
+                LOGGER.debug("Received print task: %s", print_task)
                 try:
                     self._process_print_task(print_task)
                 except Exception as e:
@@ -195,22 +175,23 @@ class PrintThread(threading.Thread):
                     # queue finish, we need to have all the print tasks
                     # finished, even if an exception happens trying to print
                     # them.
-                    self._print_queue.task_done()
+                    self._result_queue.task_done()
             except Queue.Empty:
                 pass
             if self._done.isSet():
                 break
 
     def _process_print_task(self, print_task):
-        print_str = print_task['result']
+        print_str = print_task['message']
+        if print_task['error']:
+            self.num_errors_seen += 1
         final_str = ''
-        if print_task.get('part', ''):
+        if 'total_parts' in print_task:
             # Normalize keys so failures and sucess
             # look the same.
             op_list = print_str.split(':')
             print_str = ':'.join(op_list[1:])
-            print_part = print_task['part']
-            total_part = print_part['total']
+            total_part = print_task['total_parts']
             self._num_parts += 1
             if print_str in self._progress_dict:
                 self._progress_dict[print_str]['parts'] += 1
@@ -222,8 +203,6 @@ class PrintThread(threading.Thread):
             print_components = print_str.split(':')
             final_str += print_str.ljust(self._progress_length, ' ')
             final_str += '\n'
-            if print_task.get('error', ''):
-                final_str += print_task['error'] + '\n'
             key = ':'.join(print_components[1:])
             if key in self._progress_dict:
                 self._progress_dict.pop(print_str, None)
