@@ -35,6 +35,7 @@ from awscli.customizations.sessendemail import register_ses_send_email
 from awscli.customizations.iamvirtmfa import IAMVMFAWrapper
 from awscli.customizations.argrename import register_arg_renames
 from awscli.customizations.dryrundocs import register_dryrun_docs
+from awscli.customizations.route53resourceid import register_resource_id
 
 
 def awscli_initialize(event_handlers):
@@ -73,3 +74,4 @@ def awscli_initialize(event_handlers):
     IAMVMFAWrapper(event_handlers)
     register_arg_renames(event_handlers)
     register_dryrun_docs(event_handlers)
+    register_resource_id(event_handlers)
